@@ -1,4 +1,3 @@
-import itertools
 import uuid
 from dataclasses import dataclass, field
 from typing import (
@@ -6,29 +5,28 @@ from typing import (
     TYPE_CHECKING,
     Union,
     Dict,
-    Sequence,
-    Generator,
-    Iterator,
+    Iterable,
     List,
 )
 
+import numpy as np
 from qdrant_client import QdrantClient
-from qdrant_openapi_client.models.models import (
+from qdrant_client.http.models.models import (
     Distance,
     CreateCollection,
     PointsList,
     PointStruct,
+    HnswConfigDiff,
 )
 
 from docarray import Document
 from docarray.array.storage.base.backend import BaseBackendMixin
 from docarray.array.storage.qdrant.helper import DISTANCES
 from docarray.helper import dataclass_from_dict, random_identity
+from docarray.math.helper import EPSILON
 
 if TYPE_CHECKING:
-    from docarray.types import (
-        DocumentArraySourceType,
-    )
+    from ....typing import DocumentArraySourceType, ArrayType
 
 
 @dataclass
@@ -40,6 +38,9 @@ class QdrantConfig:
     port: Optional[int] = field(default=6333)
     serialize_config: Dict = field(default_factory=dict)
     scroll_batch_size: int = 64
+    ef_construct: Optional[int] = None
+    full_scan_threshold: Optional[int] = None
+    m: Optional[int] = None
 
 
 class BackendMixin(BaseBackendMixin):
@@ -62,8 +63,6 @@ class BackendMixin(BaseBackendMixin):
             raise an error if both are provided
         """
 
-        from ... import DocumentArray
-
         self._schemas = None
 
         if not config:
@@ -105,18 +104,25 @@ class BackendMixin(BaseBackendMixin):
         # is provided, :class:`DocumentArraySqlite` will clear the existing
         # table and load the given `docs`
         self.clear()
-        if isinstance(
-            docs, (DocumentArray, Sequence, Generator, Iterator, itertools.chain)
-        ):
+        if isinstance(docs, Iterable):
             self.extend(docs)
         elif isinstance(docs, Document):
             self.append(docs)
 
     def _initialize_qdrant_schema(self):
         if not self._collection_exists(self.collection_name):
+            hnsw_config = HnswConfigDiff(
+                ef_construct=self._config.ef_construct,
+                full_scan_threshold=self._config.full_scan_threshold,
+                m=self._config.m,
+            )
             self.client.http.collections_api.create_collection(
                 self.collection_name,
-                CreateCollection(vector_size=self.n_dim, distance=self.distance),
+                CreateCollection(
+                    vector_size=self.n_dim,
+                    distance=self.distance,
+                    hnsw_config=hnsw_config,
+                ),
             )
 
     def _collection_exists(self, collection_name):
@@ -125,7 +131,7 @@ class BackendMixin(BaseBackendMixin):
         return collection_name in collections
 
     @staticmethod
-    def _qmap(doc_id: str):
+    def _map_id(doc_id: str):
         # if doc_id is a random ID in hex format, just translate back to UUID str
         # otherwise, create UUID5 from doc_id
         try:
@@ -147,11 +153,9 @@ class BackendMixin(BaseBackendMixin):
     def _get_offset2ids_meta(self) -> List[str]:
         if not self._collection_exists(self.collection_name_meta):
             return []
-        return (
-            self.client.http.points_api.get_point(self.collection_name_meta, id=1)
-            .result.payload['offset2id']
-            .value
-        )
+        return self.client.http.points_api.get_point(
+            self.collection_name_meta, id=1
+        ).result.payload['offset2id']
 
     def _update_offset2ids_meta(self):
         if not self._collection_exists(self.collection_name_meta):
@@ -161,7 +165,7 @@ class BackendMixin(BaseBackendMixin):
             )
 
         self.client.http.points_api.upsert_points(
-            name=self.collection_name_meta,
+            collection_name=self.collection_name_meta,
             wait=True,
             point_insert_operations=PointsList(
                 points=[
@@ -172,14 +176,17 @@ class BackendMixin(BaseBackendMixin):
             ),
         )
 
-    def _get_storage_infos(self) -> Dict:
-        storage_infos = super()._get_storage_infos()
-        return {
-            'Backend': 'Qdrant (https://qdrant.tech)',
-            'Host': self._config.host,
-            'Port': str(self._config.port),
-            'Collection Name': self.collection_name,
-            'Distance': self._config.distance,
-            'Serialization Protocol': self._config.serialize_config.get('protocol'),
-            **storage_infos,
-        }
+    def _map_embedding(self, embedding: 'ArrayType') -> List[float]:
+        if embedding is None:
+            embedding = np.random.rand(self.n_dim)
+        else:
+            from ....math.ndarray import to_numpy_array
+
+            embedding = to_numpy_array(embedding)
+
+        if embedding.ndim > 1:
+            embedding = np.asarray(embedding).squeeze()
+
+        if np.all(embedding == 0):
+            embedding = embedding + EPSILON
+        return embedding.tolist()
